@@ -1,17 +1,10 @@
 #define IN_LIBEXSLT
 #include "libexslt/libexslt.h"
 
-#if defined(WIN32) && !defined (__CYGWIN__) && (!__MINGW32__)
-#include <win32config.h>
-#else
-#include "config.h"
-#endif
-
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
-#include <libxslt/xsltconfig.h>
 #include <libxslt/xsltutils.h>
 #include <libxslt/xsltInternals.h>
 #include <libxslt/extensions.h>
@@ -298,7 +291,7 @@ static void
 exsltMathLowestFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlNodeSetPtr ns, ret;
     void *user = NULL;
-    
+
 
     if (nargs != 1) {
 	xmlXPathSetArityError(ctxt);
@@ -368,7 +361,7 @@ exsltMathConstant (xmlChar *name, double precision) {
 
         if (precision <= len)
             len = (int)precision;
-        
+
         str = xmlStrsub(EXSLT_PI, 0, len);
 
     } else if (xmlStrEqual(name, BAD_CAST "E")) {
@@ -376,7 +369,7 @@ exsltMathConstant (xmlChar *name, double precision) {
 
         if (precision <= len)
             len = (int)precision;
-        
+
         str = xmlStrsub(EXSLT_E, 0, len);
 
     } else if (xmlStrEqual(name, BAD_CAST "SQRRT2")) {
@@ -384,7 +377,7 @@ exsltMathConstant (xmlChar *name, double precision) {
 
         if (precision <= len)
             len = (int)precision;
-        
+
         str = xmlStrsub(EXSLT_SQRRT2, 0, len);
 
     } else if (xmlStrEqual(name, BAD_CAST "LN2")) {
@@ -392,7 +385,7 @@ exsltMathConstant (xmlChar *name, double precision) {
 
         if (precision <= len)
             len = (int)precision;
-        
+
         str = xmlStrsub(EXSLT_LN2, 0, len);
 
     } else if (xmlStrEqual(name, BAD_CAST "LN10")) {
@@ -400,7 +393,7 @@ exsltMathConstant (xmlChar *name, double precision) {
 
         if (precision <= len)
             len = (int)precision;
-        
+
         str = xmlStrsub(EXSLT_LN10, 0, len);
 
     } else if (xmlStrEqual(name, BAD_CAST "LOG2E")) {
@@ -408,7 +401,7 @@ exsltMathConstant (xmlChar *name, double precision) {
 
         if (precision <= len)
             len = (int)precision;
-        
+
         str = xmlStrsub(EXSLT_LOG2E, 0, len);
 
     } else if (xmlStrEqual(name, BAD_CAST "SQRT1_2")) {
@@ -416,7 +409,7 @@ exsltMathConstant (xmlChar *name, double precision) {
 
         if (precision <= len)
             len = (int)precision;
-        
+
         str = xmlStrsub(EXSLT_SQRT1_2, 0, len);
 
     } else {
